
install.packages(c("ape","geiger","phylotools","adephylo","Hmisc",
                   "RRphylo", "devtools", "ape", "rlg","phyloextra",
                   "nlme","picante","SYNCSA", "MuMIn", "TreeSim",
                   "diversitree","RPANDA","PCPS","maptools","rgdal",
                   "ggplot2","wesanderson","RColorBrewer","graphics",
                   "matlab","gmt","letsR","sf","lavaan","semPlot",
                   "qpcR","semTools"))



#Pacotes necessarios
library(ape)
library(geiger)
library(phylotools)
library(adephylo)
library(Hmisc)
library(rgl)
library(RRphylo)
library(devtools)
install_github("liamrevell/phytools")
library(phytools)

library(nlme)
library(picante)
library(SYNCSA)
library(MuMIn)
library(TreeSim)
library(diversitree)
library(RPANDA)




#######################################################################
#Ler filogenia felidae


tree=read.tree("phylo_felidae.txt")

plot(tree)


#######################################################################
##  RESOLVENDO POLITOMIAS ALEATORIAMENTE


tree=fix.poly(tree,type=c("resolve"),node=NULL,tol=1e-10,random=TRUE)

plot(tree)

axisPhylo(side = 1, root.time = NULL, backward = TRUE)





#######################################################################
##LENDO OS DADOS

dados<-read.table("felidae_traits.txt",h=T,row.names=1)
head(dados)


match.species<-treedata(tree,dados)
tree<-match.species$phy
plotTree(tree,fsize=0.5,ftype="i",type="fan",lwd=1)


#######################################################################
## RECOSNTRUÇÃO TRAIT CONTINUO

massa<-log(dados$Body_Mass_g)
names(massa)=rownames(dados)
massa

set.seed(13)

obj<-contMap(tree,massa,plot=FALSE)

contMap_massa<-setMap(obj,hcl.colors(10,
                                   palette="Spectral",rev=TRUE))


plot(contMap_massa,legend=0.6 *max(nodeHeights(tree)),
     fsize=c(0.7,0.8),invert=TRUE)

#aumento de peso ocorreu duas vezes de forma independente 


###projetando os dados no espaco filogenetico 
phenogram(tree,massa,fsize=0.6,spread.costs=c(1,0))




#######################################################################
# identificando locais de maior incerteza

fancyTree(tree,type="phenogram95",x=massa,spread.cost=c(1,0))
#pode levar alguns minutos 


#podemos compensar com dados faltantes inserindo aos nos da filogenia 


#######################################################################
## RECOSNTRUÇÃO TRAIT CATEGORICO 


pelagem<-dados$pelagem
names(pelagem)=rownames(dados)
pelagem


plotTree(tree,fsize=0.7,ftype="i")


cols<-setNames(palette()[1:length(unique(pelagem))],sort(unique(pelagem)))
tiplabels(pie=to.matrix(pelagem,sort(unique(pelagem))),piecol=cols,cex=0.2)
add.simmap.legend(colors=cols,prompt=T,x=0.2,
                  y=-max(nodeHeights(tree)),fsize=0.8)

set.seed(13)

## estimate ancestral states 
fitER<-ace(pelagem,tree,model="ER",type="discrete")
fitER


plotTree(tree,fsize=0.7,ftype="i")


nodelabels(node=1:tree$Nnode+Ntip(tree),
           pie=fitER$lik.anc,piecol=cols,cex=0.5)
tiplabels(pie=to.matrix(pelagem,sort(unique(pelagem))),piecol=cols,cex=0.3)
add.simmap.legend(colors=cols,prompt=T,x=0.2,
                  y=-max(nodeHeights(tree)),fsize=0.8)


#######################################################################
#deixando mais complexo fazendo com 100 arvores e tirando media 

set.seed(13)

mtrees<-make.simmap(tree,pelagem,model="ER",nsim=100)

par(mfrow=c(10,10))
null<-sapply(mtrees,plot,colors=cols,lwd=1,ftype="off")


pd<-summary(mtrees,plot=FALSE)
pd

plot(pd,fsize=0.6,ftype="i")

add.simmap.legend(colors=cols,prompt=T,x=0.2,
                  y=-max(nodeHeights(tree)),fsize=0.8)



#######################################################################
#phylomorfoespaco - vizualizacao em duas ou mais dimensoes 

#vamos criar uma subtabela com 3 atributos
#massa corportal 
#tamanho corporal
#tamanho de range


data_test=data.frame(dados[3],dados[4], dados[6])
data_test


#como sao valores muito discrepantes, vamos tirar o log pra comparar 


data_test=log(data_test)
data_test


#aqui vamos faer apenas com massa e tamano (duas dimensoes)
phylomorphospace(tree,data_test[,c(1,2)],xlab="Massa",ylab="Tamanho", fsize=0.7)


#agora podemos fazer com 3 dimesoes uma versao animada 
phylomorphospace3d(tree,data_test,method = "dynamic", fsize=0.1)




#######################################################################
#evolucao espelhada entre massa e tamanho

par(mfrow=c(1,2))


####reconstruindo a massa corporal
massa<-log(dados$Body_Mass_g)
names(massa)=rownames(dados)
massa

set.seed(13)

obj<-contMap(tree,massa,plot=FALSE)

contMap_massa<-setMap(obj,hcl.colors(10,
                                     palette="Spectral",rev=TRUE))


plot(contMap_massa,legend=0.6 *max(nodeHeights(tree)),
     fsize=c(0.7,0.8),invert=TRUE)


######agora reconstruimos o tamanho, mas mudamos a orientacao do plot 

tamanho<-log(dados$Body_length_mm)
names(tamanho)=rownames(dados)
tamanho

set.seed(13)

obj<-contMap(tree,tamanho,plot=FALSE)

contMap_tamanho<-setMap(obj,hcl.colors(10,
                                     palette="Spectral",rev=TRUE))


plot(contMap_tamanho,legend=0.6 *max(nodeHeights(tree)),
     fsize=c(0.7,0.8),invert=TRUE, direction="leftwards")





#######################################################################
#evolucao espelhada entre tamanho e longevidade 

par(mfrow=c(1,2))


####reconstruindo a longevidade
longevidade<-log(dados$Longevity_months)
names(longevidade)=rownames(dados)
longevidade

set.seed(13)

obj<-contMap(tree,longevidade,plot=FALSE)

contMap_longevidade<-setMap(obj,hcl.colors(10,
                                     palette="Spectral",rev=TRUE))


plot(contMap_longevidade,legend=0.6 *max(nodeHeights(tree)),
     fsize=c(0.7,0.8),invert=TRUE)


######agora reconstruimos o tamanho 

tamanho<-log(dados$Body_length_mm)
names(tamanho)=rownames(dados)
tamanho

set.seed(13)

obj<-contMap(tree,tamanho,plot=FALSE)

contMap_tamanho<-setMap(obj,hcl.colors(10,
                                       palette="Spectral",rev=TRUE))


plot(contMap_tamanho,legend=0.6 *max(nodeHeights(tree)),
     fsize=c(0.7,0.8),invert=TRUE, direction="leftwards")



#######################################################################
#######################################################################
#############                                                 #########
#############  CORRELACAO DE ATRIBUTO - MODELOS EVOLUTIVOS    ######### 
#############   considerar a nao independencia entre sps      #########
#############                                                 #########
#######################################################################
#######################################################################


#atributos entre especies irmas nao sao independentes
#pgls corrige isso

#--------------------------------------------------------------------------------------------------
##PGLS   Tamanho X Longevidade
#--------------------------------------------------------------------------------------------------

##corBrowniano 
tamanho_x_longevidade_brow<- gls(tamanho ~ longevidade, correlation = corBrownian(phy = tree),
                        data = dados, method = "ML")
summary(tamanho_x_longevidade_brow)
coef(tamanho_x_longevidade_brow)
str(tamanho_x_longevidade_brow) 
plot(tamanho ~ longevidade,pch=19)
abline(a = coef(tamanho_x_longevidade_brow)[1], b = coef(tamanho_x_longevidade_brow)[2], col=c("red")) 



##corMartins - OU
tamanho_x_longevidade_ou <- gls(tamanho ~ longevidade, correlation=corMartins(value=1,phy=tree),
                       data = dados, method = "ML")
summary(tamanho_x_longevidade_ou)
coef(tamanho_x_longevidade_ou)
plot(tamanho ~ longevidade,pch=19)
abline(a = coef(tamanho_x_longevidade_ou)[1], b = coef(tamanho_x_longevidade_ou)[2], col=c("red")) 


##corBlomberg - ACDC
tamanho_x_longevidade_blomberg <- gls(tamanho ~ longevidade,correlation= corBlomberg ( fixed = T,value=0.5,phy=tree),
                             data = dados, method = "ML")
summary(tamanho_x_longevidade_blomberg)
coef(tamanho_x_longevidade_blomberg)
plot(tamanho ~ longevidade,pch=19)
abline(a = coef(tamanho_x_longevidade_blomberg)[1], b = coef(tamanho_x_longevidade_blomberg)[2], col=c("red")) 



##corPagel
tamanho_x_longevidade_pagel <- gls(tamanho ~ longevidade,correlation= corPagel (value=1,phy=tree),
                          data = dados, method = "REML")
summary(tamanho_x_longevidade_pagel)
coef(tamanho_x_longevidade_pagel)
plot(tamanho ~ longevidade,pch=19)
abline(a = coef(tamanho_x_longevidade_pagel)[1], b = coef(tamanho_x_longevidade_pagel)[2], col=c("red")) 



####SELECAO DOS MODELOS DE PGLS 
best_model <- (msAICc <- model.sel(tamanho_x_longevidade_pagel,tamanho_x_longevidade_blomberg,
                                                tamanho_x_longevidade_ou,tamanho_x_longevidade_brow))
best_model




#######################################################################
#######################################################################
#############                                                 #########
#############           EXPLORAR DADOS DA FILOGENIA           #########
#############                                                 #########
#######################################################################
#######################################################################


##LTT plots (lineages through time)

ltt(tree)


##podemos ver a linhagens em separada para cada estado de atributo 
##vamos voltar ao exemplo de pelagem

#vamos separar todas as especies que tenham cada um dos 6 estados 

species_listras=c("Felis_bieti",
                   "Felis_margarita",
                   "Felis_silvestris",
                   "Oreailurus_jacobita",
                   "Otocolobus_manul",
                   "Panthera_tigris")


species_mancha=c("Acinonyx_jubatus",
                 "Felis_nigripes",
                 "Leptailurus_serval",
                 "Lynx_canadensis",
                 "Lynx_lynx",
                 "Lynx_pardinus",
                 "Lynx_rufus",
                 "Oncifelis_geoffroyi",
                 "Oncifelis_guigna",
                 "Prionailurus_bengalensis",
                 "Prionailurus_rubiginosus",
                 "Prionailurus_viverrinus",
                 "Profelis_aurata")


species_mancha_2=c("Leopardus_pardalis",
                   "Leopardus_tigrinus",
                   "Leopardus_wiedii")


species_mancha_3=c("Neofelis_nebulosa",
                   "Oncifelis_colocolo",
                   "Pardofelis_marmorata")

species_rosetas=c("Panthera_onca",
                  "Panthera_pardus",
                  "Uncia_uncia")


species_uniforme=c("Caracal_caracal",
                   "Catopuma_badia",
                   "Catopuma_temminckii",
                   "Felis_chaus",
                   "Herpailurus_yaguarondi",
                   "Panthera_leo",
                   "Prionailurus_planiceps",
                   "Puma_concolor")



#filogenia especies com manchas
comparison_mancha<-name.check(tree,species_mancha,data.names=species_mancha)
comparison_mancha
comparison_mancha$tree_not_data
tree_mancha<-drop.tip(tree,comparison_mancha$tree_not_data)
tree_mancha
plotTree(tree_mancha, fsize=1, lwd=1)


#filogenia especies com listras
comparison_listrada<-name.check(tree,species_listras,data.names=species_listras)
comparison_listrada
comparison_listrada$tree_not_data
tree_listrada<-drop.tip(tree,comparison_listrada$tree_not_data)
tree_listrada
plotTree(tree_listrada, fsize=1, lwd=1)


#filogenia especies com mancha_2
comparison_mancha_2<-name.check(tree,species_mancha_2,data.names=species_mancha_2)
comparison_mancha_2
comparison_mancha_2$tree_not_data
tree_mancha_2<-drop.tip(tree,comparison_mancha_2$tree_not_data)
tree_mancha_2
plotTree(tree_mancha_2, fsize=1, lwd=1)


#filogenia especies com mancha_3
comparison_mancha_3<-name.check(tree,species_mancha_3,data.names=species_mancha_3)
comparison_mancha_3
comparison_mancha_3$tree_not_data
tree_mancha_3<-drop.tip(tree,comparison_mancha_3$tree_not_data)
tree_mancha_3
plotTree(tree_mancha_3, fsize=1, lwd=1)


#filogenia especies com rosetas
comparison_rosetas<-name.check(tree,species_rosetas,data.names=species_rosetas)
comparison_rosetas
comparison_rosetas$tree_not_data
tree_rosetas<-drop.tip(tree,comparison_rosetas$tree_not_data)
tree_rosetas
plotTree(tree_rosetas, fsize=1, lwd=1)


#filogenia especies com uniforme
comparison_uniforme<-name.check(tree,species_uniforme,data.names=species_uniforme)
comparison_uniforme
comparison_uniforme$tree_not_data
tree_uniforme<-drop.tip(tree,comparison_uniforme$tree_not_data)
tree_uniforme
plotTree(tree_uniforme, fsize=1, lwd=1)




#arvore inteira
obj1<-ltt(tree,show.tree=TRUE,col="red", gamma = TRUE,lwd=2, log.lineages=T,las=1,ylim=c(0,3),xaxt = "n")
axisPhylo()

#manchas
obj2<-ltt(tree_mancha,show.tree=TRUE,col="yellow", gamma = TRUE,lwd=2, log.lineages=T,las=1,ylim=c(0,3),xaxt = "n")
axisPhylo()

#listras
obj3=ltt(tree_listrada,show.tree=TRUE,col="green",gamma = TRUE,lwd=2, log.lineages=T,las=1, ylim=c(0,3),xaxt = "n")
axisPhylo()

#mancha_2
obj4<-ltt(tree_mancha_2,show.tree=TRUE,col="blue",lwd=2,gamma = TRUE, log.lineages=T,las=1, ylim=c(0,3),xaxt = "n")
axisPhylo()

#mancha_3
obj5<-ltt(tree_mancha_3,show.tree=TRUE,gamma = TRUE,col="black",lwd=2, log.lineages=T,las=1,xaxt = "n")
axisPhylo()

#uniforme 
obj6<-ltt(tree_uniforme,show.tree=TRUE,gamma = TRUE,col="purple",lwd=2, log.lineages=T,las=1,xaxt = "n")
axisPhylo()

#rosetas
obj7<-ltt(tree_rosetas,show.tree=TRUE,gamma = TRUE,col="orange",lwd=2, log.lineages=T,las=1,xaxt = "n")
axisPhylo()



######LTTs

#arvore interira 
obj1<-ltt(tree,show.tree=TRUE,log.lineages=T,col="red",lwd=2,las=1,xaxt = "n")
axisPhylo()


#cada um dos estados 
obj2<-ltt(tree_mancha,log.lineages=T,col="yellow",lwd=2,las=1,xaxt = "n")
obj3<-ltt(tree_listrada,log.lineages=T,add=T,col="green",lwd=2)
obj4<-ltt(tree_mancha_2,log.lineages=T,add=T,col="blue",lwd=2)
obj5<-ltt(tree_mancha_3,log.lineages=T,add=T,col="black",lwd=2)
obj6<-ltt(tree_uniforme,log.lineages=T,add=T,col="purple",lwd=2)
obj7<-ltt(tree_rosetas,log.lineages=T,add=T,col="orange",lwd=2)

axisPhylo()





#######################################################################
#######################################################################
#############                                                 #########
#############       SINAL FILOGENETICO - ESTATISTICA K        #########
#############                                                 #########
#######################################################################
#######################################################################


#Sinal filogen3tico = meramente um padrAo que mostra se espEcies mais prOximas
#têm valores de atributo mais similares do que espEcies mais distantes. 
#O mEtodos mais comum = estatística K (Blomberg et al 2003) 

#sempre comparamos com o movimento browniano

#K=1 indica que espécies próximas são tão parecidas quanto prevê o modelo Browniano
#K<1 indica sinal filogenético menor do que esperado pelo modelo Browniano
#K>1 indica sinal filogenético maior do que esperado pelo modelo Browniano

# K pra tamanho corporal
phylosig(tree,dados$Body_length_mm,method="K")

# K pra massa corporal
phylosig(tree,dados$Body_Mass_g,method="K")



#######################################################################
#######################################################################
#############                                                 #########
#############           TAXAS DE ESPECIACAO E EXTINCAO        #########
#############                                                 #########
#######################################################################
#######################################################################



#####Modelo pure-birth (sem extinxao)

#simular mil filogenias atraves desse modelo 

allTheTrees <- sim.bd.taxa.age(n = 109, numbsim = 1000, lambda = 0.4, mu = 0,
                               age = 10, mrca = T)

ltt.plot(allTheTrees[[1]], log = "y")
for (i in 2:1000) {
  ltt.lines(allTheTrees[[i]])
}


#######################################################################


#Modelo Birth-death 

#simular mil filogenias atraves desse modelo 

allTheTrees2 <- sim.bd.taxa.age(n = 109, numbsim = 1000, lambda = 2, mu = 1.6,
                                age = 10, mrca = T)

ltt.plot(allTheTrees2[[1]], log = "y")
for (i in 2:1000) {
  ltt.lines(allTheTrees2[[i]])
}


#######################################################################

#simulando as filogenias nesses modelos 

simTree1 <- sim.bd.taxa.age(n = 109, age = 10, numbsim = 1, lambda = 0.4, mu = 0)[[1]]
simTree2 <- sim.bd.taxa.age(n = 109, age = 10, numbsim = 1, lambda = 2, mu = 1.6)[[1]]

par(mfcol = c(1, 2))
plot(simTree1, main = "Pure Birth")
plot(simTree2, main = "Birth-death")



#######################################################################

### calulando especiacao e extincao em nossa filogenia 
#comparando entre modelos pure-birth e birth-death


### Yule (pure-birth -- somente especiação)
modelo.pb<-make.yule(tree,sampling.f = 36/38)
param.pb<-find.mle(modelo.pb,x.init=0.1) 
#x.init é um valor arbitrário para iniciar o otimização

param.pb

param.pb$par #lambda=especiacao



### Modelo birth-death (especiação + extinção)
modelo.bd<-make.bd(tree,sampling.f = 36/38)
param.bd<-find.mle(modelo.bd,x.init=c(0.1,0.05))

param.bd

param.bd$par#lambda=especiacao, mu=extincao




#MCMC.
# tambem podemos rodar analises bayesianas para estimar os parâmetros de
#diversificação usando a distribuição posterior probabilidades.

bd.amostras <- mcmc(modelo.bd, param.bd$par, nsteps = 1e+05, lower = c(0, 0),
                    upper = c(Inf,Inf), w = c(0.1, 0.1), fail.value = -Inf,
                    print.every = 10000)
# visualizar distribuição
postSamples<-bd.amostras[c("lambda", "mu")]
par(mfrow=c(1,1),mar=c(5,4,2,2))
profiles.plot(postSamples,col.line=c("red", "blue"),las=1,legend ="topright")






#######################################################################
#######################################################################
#############                                                 #########
#############             DIVERSIFICACAO VARIAVEL             #########
#############                                                 #########
#######################################################################
#######################################################################


#Diversificação Time-dependent
#RPANDA é um pacote (Morlon et al 2016) que permite ajustar modelos com
#taxas de especiação e/ou extinção variáveis, e comparar os 
#modelos usando likelihood


lambda.cst <- function(t,y){y[1]} # lambda constante
lambda.var <- function(t,y){y[1]*exp(y[2]*t)} # lambda com variação exponencial (emfunção do tempo t)
mu.cst <- function(t,y){y[1]}
mu.var <- function(t,y){y[1]*exp(y[2]*t)}


# Ajustando modelos para toda a árvore


# birth-death especiação e extinção constante
bcdc<-
  fit_bd(tree,tot_time=max(branching.times(tree)),f.lamb=lambda.cst,f.mu=mu.cst,
         lamb_par=0.01,mu_par=0.001,cst.lamb=TRUE,cst.mu=TRUE)
bcdc$model<-"birth-death taxa de especiação e extinção constantes"
bcdc


# birth-death especiação exponencial e extinção constante
bvdc<-
  fit_bd(tree,tot_time=max(branching.times(tree)),f.lamb=lambda.var,f.mu=mu.cst,
         lamb_par=c(0.05,0.01),mu_par=0.001,expo.lamb=TRUE,cst.mu=TRUE)
bvdc$model<-"birth-death taxa de especiação exponencial e extinção constante"


# birth-death especiação constante e extinção exponencial
bcdv<-
  fit_bd(tree,tot_time=max(branching.times(tree)),f.lamb=lambda.cst,f.mu=mu.var,
         lamb_par=0.01,mu_par=c(0.005,0.001),cst.lamb=TRUE,expo.mu=TRUE)
bcdv$model<-"birth-death taxa de especiação constante e extinção exponencial"


# birth-death especiação e extinção exponencial
bvdv<-
  fit_bd(tree,tot_time=max(branching.times(tree)),f.lamb=lambda.var,f.mu=mu.var,
         lamb_par=c(0.05,0.01),mu_par=c(0.005,0.001),expo.lamb=TRUE,expo.mu=TRUE)
bvdv$model<-"birth-death taxa de especiação e extinção exponencial"




# Comparando modelos
aic.scores<-setNames(c(bcdc$aicc,bvdc$aicc,bcdv$aicc,bvdv$aicc),
c("bcdc","bvdc","bcdv","bvdv"))
aic.scores
aic.w(aic.scores)




#######################################################################
#######################################################################
#############                                                 #########
#############  TRANSICOES ENTRE ESTADOS AO LONGO DA EVOLUCAO  #########
#############                                                 #########
#######################################################################
#######################################################################


pelagem_abreviacao<-dados$pelagem_abrev
names(pelagem_abreviacao)=rownames(dados)
pelagem_abreviacao


## fit ARD model
fitARD<-fitMk(tree,pelagem_abreviacao,model="ARD",pi="fitzjohn")
fitARD
#vai demorar alguns minutos


## create plot
plot(fitARD,color=TRUE,lwd=3)
title(main="Mudança de estados ao longo da evolução",
      font.main=3,line=-1)

